/* Copyright (C) 1991, 92, 95, 96, 97, 98 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifdef DEBUG
#include <stdio.h>
#endif
#include <stdlib.h>
#include <wchar.h>
#include <gconv.h>
#include "wcsmbsload.h"

extern int mbstowcs_savestate;

extern mbstate_t __no_r_state;	/* Defined in mbtowc.c.  */

/* Convert WCHAR into its multibyte character representation,
   putting this in S and returning its length.

   Attention: this function should NEVER be intentionally used.
   The interface is completely stupid.  The state is shared between
   all conversion functions.  You should use instead the restartable
   version `wcrtomb'.  */
int
wctomb (char *s, wchar_t wchar)
{
  int result;

#ifdef DEBUG
  fprintf(stderr, "begin: wctomb\n");
  fprintf(stderr, "info: char  s       = [%s]\n", s ? "(something)" : "(null)");
  fprintf(stderr, "info: char  s(p)    = [%p]\n", s);
  fprintf(stderr, "info: wchar wchar(p)= [%p]\n", &wchar);            
#endif

  /* If S is NULL the function has to return null or not null
     depending on the encoding having a state depending encoding or
     not.  */
  if (s == NULL)
    {
      /* Make sure we use the correct value.  */
      update_conversion_ptrs ();

      result = __wcsmbs_gconv_fcts.tomb->stateful;
#ifdef DEBUG
      fprintf(stderr, "info: return %d(wctomb)\n", result);            
#endif
      return result;
    }

  if ( 1 == mbstowcs_savestate) {
    mbstowcs_savestate = 2;
#ifdef DEBUG
    fprintf(stderr, "info: *** mbstowcs_savestate is [%d] ***\n",
            mbstowcs_savestate);
#endif
  } else {
    mbstowcs_savestate = 0;
  }

  result = wcrtomb (s, wchar, &__no_r_state);

#ifdef DEBUG
  fprintf(stderr, "info: char  s     = [%s](after)\n", s ? s : "(null)");
  fprintf(stderr, "info: return %d(wctomb)\n", result);            
#endif
  return result;
}
